const { createMacro } = require('babel-plugin-macros');
const template = require('@babel/template');

function createGraphql({ path, t }, query, name, rest = []) {
  return t.callExpression(
    path.scope.bindings.graphql
      ? path.scope.bindings.graphql.identifier
      : t.toExpression(
          template.default(`require("@apollo/react-hoc").graphql`)()
        ),
    [
      query,
      t.objectExpression([
        t.objectProperty(t.identifier('name'), name),
        ...rest
      ])
    ]
  );
}

module.exports = createMacro(({ references, state, babel }) => {
  const t = babel.types;
  references.default.forEach(path => {
    if (path.parentPath && t.isCallExpression(path.parentPath)) {
      const component = path.parentPath.get('arguments')[0].node;
      console.log(component);
      let graphqls = [];
      path.parentPath.parentPath.container.forEach((node, i) => {
        if (t.isExportNamedDeclaration(node)) {
          if (node.declaration) {
            const query = node.declaration.declarations[0].init;
            let queryArgs, graphqlQuery, graphqlConfig;
            if (t.isArrayExpression(query)) {
              // with config
              [graphqlQuery, graphqlConfig] = query.elements;
            } else {
              // no config
              graphqlQuery = query;
            }

            if (
              (t.isTaggedTemplateExpression(graphqlQuery) &&
                (graphqlQuery.tag.name === 'gql' ||
                  graphqlQuery.tag.name === 'graphql')) ||
              (t.isCallExpression(graphqlQuery) &&
                (graphqlQuery.callee.name === 'gql' ||
                  graphqlQuery.callee.name === 'graphql'))
            ) {
              queryArgs = [graphqlQuery, graphqlConfig];
            }
            if (queryArgs) {
              // matched graphql query
              const name = node.declaration.declarations[0].id.name;
              graphqls.push(
                createGraphql(
                  { path: path.parentPath.parentPath, t },
                  queryArgs[0],
                  t.stringLiteral(name),
                  queryArgs[1] ? queryArgs[1].properties : []
                )
              );
              path.scope.bindings[name].references < 2 &&
                (path.container[i] = null);
            }
          } else if (
            node.source &&
            /\.(gql|graphql)$/.test(node.source.value)
          ) {
            node.specifiers.forEach(specifier => {
              graphqls.push(
                createGraphql(
                  { path: path.parentPath.parentPath, t },
                  t.toExpression(
                    template.default(
                      `require('${node.source.value}').${specifier.exported.name}`
                    )()
                  ),
                  t.stringLiteral(specifier.exported.name)
                )
              );
            });
          }
        }
      });
      if (graphqls.length) {
        path.parentPath.replaceWith(
          t.callExpression(
            graphqls.length > 1
              ? t.callExpression(
                  path.scope.bindings.compose
                    ? path.scope.bindings.compose.identifier
                    : t.toExpression(
                        template.default(`require("lodash/flowRight")`)()
                      ),
                  graphqls
                )
              : graphqls[0],
            [component]
          )
        );
      }
    }
  });
});
